<?php

/**
 * @file
 * Administrative form for QuickBooks settings.
 */

/**
 * Builds the main settings page.
 */
function ji_quickbooks_admin_main_settings() {
  // If all goes well, ji-quickbooks.joshideas.com will return the
  // what we need.
  if (filter_input(INPUT_POST, 'realm_id')) {
    variable_set('ji_quickbooks_settings_realm_id', filter_input(INPUT_POST, 'realm_id'));
  }

  if (filter_input(INPUT_POST, 'oauth_token')) {
    variable_set('ji_quickbooks_settings_access_token', filter_input(INPUT_POST, 'oauth_token'));
  }

  if (filter_input(INPUT_POST, 'oauth_token_secret')) {
    variable_set('ji_quickbooks_settings_access_token_secret', filter_input(INPUT_POST, 'oauth_token_secret'));
  }

  if (filter_input(INPUT_POST, 'consumer_key')) {
    variable_set('ji_quickbooks_settings_consumer_key', filter_input(INPUT_POST, 'consumer_key'));
  }

  if (filter_input(INPUT_POST, 'consumer_secret')) {
    variable_set('ji_quickbooks_settings_consumer_secret', filter_input(INPUT_POST, 'consumer_secret'));
  }

  // If the module is not yet installed guide them to the right path.
  $ji_quickbooks = ji_quickbooks_get_path('qbosdk3260');
  if (empty($ji_quickbooks)) {
    $link_placeholder = l(t('here'), '/admin/help/ji_quickbooks');
    drupal_set_message(t("The QuickBooks SDK is missing. Please install that first. Click $link_placeholder for more information."), 'error', FALSE);

    $form['ji_quickbooks'] = array(
      '#type' => 'fieldset',
      '#title' => t('QuickBooks Settings'),
      '#description' => t("Please follow these steps in order to complete the installation:"),
      '#collapsible' => FALSE,
      '#tree' => TRUE,
    );
    $list_items = array(
      'Download SDK from ' . l(t('Josh Ideas'), 'https://joshideas.com/sdk/ji_quickbooks/qbosdk3260.zip')
      . ' and place in the libraries directory (/sites/all/libraries/).',
      'The installation of OAuth on your web server using the php.ini extension=oauth.so is required. '
      . "The module will alert you if it detects OAuth isn't installed on the web server.<br>",
    );

    $form['ji_quickbooks']['payment_method'] = array(
      '#prefix' => '<ul><li>',
      '#markup' => implode('</li><li>', $list_items),
      '#suffix' => '</li></ul>',
    );
    return $form;
  }

  // Won't load the screen if OAuth isn't detected.
  module_load_include('inc', 'ji_quickbooks', 'ji_quickbooks_support');
  if (!ji_quickbooks_support_check_oauth(TRUE)) {
    return;
  }

  $quickbooks_service = new JIQuickBooksService(FALSE);
  if (!empty($quickbooks_service->settingErrorMessage)) {
    // We don't display a message since we display
    // the QBO connect buttons below.
  }
  else {
    // We retrieve only the accounts of type 'Other Current Asset' and 'Bank'
    // as per documentation QBO does allow ony those 2 types for payment
    // see https://developer.intuit.com/docs/api/accounting/payment.
    $account_array['accounts'] = $quickbooks_service->getAcountsByType(array('Other Current Asset', 'Bank'));
    $account_array['payment_methods'] = $quickbooks_service->getAllPaymentMethods();
    $account_array['terms'] = $quickbooks_service->getAllTerms();
    $account_array['products'] = $quickbooks_service->getAllProducts();

    $error_data = $quickbooks_service->checkErrors();
    $accounts = array();
    $accounts[0] = 'Please select...';

    $pay_methods = array();
    $pay_methods[0] = 'Please select...';

    $term = array();
    $term[0] = 'Please select...';

    $products = array();
    $products[0] = 'Please select...';

    if (empty($error_data['code'])) {
      foreach ($account_array['accounts'] as $account) {
        $accounts[$account->Id] = $account->Name;
      }
      foreach ($account_array['payment_methods'] as $pay_method) {
        $pay_methods[$pay_method->Id] = $pay_method->Name;
      }
      foreach ($account_array['terms'] as $terms) {
        $term[$terms->Id] = $terms->Name;
      }
      foreach ($account_array['products'] as $product) {
        $products[$product->Id] = $product->Name;
      }
    }

    $form['ji_quickbooks'] = array(
      '#type' => 'fieldset',
      '#title' => t('QuickBooks Settings'),
      '#description' => t("These are pulled from QuickBooks and during checkout, are applied to invoices and payments."),
      '#collapsible' => FALSE,
      '#tree' => TRUE,
    );
    $form['ji_quickbooks']['payment_method'] = array(
      '#type' => 'select',
      '#title' => t('Payment Method'),
      '#description' => t("When a payment is created; which method should we tell QBO is used? Hint: Probably Credit Card."),
      '#options' => $pay_methods,
      '#default_value' => variable_get('ji_quickbooks_payment_method', 0),
      '#required' => TRUE,
    );
    $form['ji_quickbooks']['account_payment'] = array(
      '#type' => 'select',
      '#title' => t('Account Selected'),
      '#description' => t("When payment is created, to what business account should QuickBooks apply it to?"),
      '#options' => $accounts,
      '#default_value' => variable_get('ji_quickbooks_payment_account', 0),
      '#required' => TRUE,
    );
    $form['ji_quickbooks']['terms'] = array(
      '#type' => 'select',
      '#title' => t('Terms'),
      '#description' => t("Choose the due date for an invoice. Currently, payments are applied immediately after invoice is created."),
      '#options' => $term,
      '#default_value' => variable_get('ji_quickbooks_term', 0),
      '#required' => TRUE,
    );
    $form['ji_quickbooks']['product'] = array(
      '#type' => 'select',
      '#title' => t('Products'),
      '#description' => t("Which QuickBooks Products or Services should all invoice line items relate to?"),
      '#options' => $products,
      '#default_value' => variable_get('ji_quickbooks_default_product', 0),
      '#required' => TRUE,
    );
  }

  $form['ji_quickbooks_config'] = array(
    '#type' => 'fieldset',
    '#title' => t('QuickBooks API Configuration'),
    '#description' => t("Please select the environment you want to connect to and press 'Save settings'. You can then 'Connect to QuickBooks'."),
    '#collapsible' => TRUE,
    '#collapsed' => !empty(variable_get('ji_quickbooks_settings_access_token', '')),
    '#tree' => TRUE,
  );

  $environments = array('dev' => 'Development Environment', 'pro' => 'Production Environment');
  $form['ji_quickbooks_config']['environment'] = array(
    '#type' => 'radios',
    '#title' => t('Environment'),
    '#default_value' => variable_get('ji_quickbooks_settings_environment', NULL),
    '#options' => $environments,
    '#required' => TRUE,
  );

  $connect_disabled = TRUE;
  // If realm_id is missing but consumer key and secret aren't, then
  // display 'Connect to QuickBooks' button.
  if (empty(variable_get('ji_quickbooks_settings_realm_id', '')) &&
      empty(variable_get('ji_quickbooks_settings_consumer_key', '')) &&
      empty(variable_get('ji_quickbooks_settings_consumer_secret', ''))) {
    $connect_disabled = FALSE;
  }  

  $form['ji_quickbooks_config']['connect'] = array(
    '#type' => 'submit',
    '#value' => t('Connect to QuickBooks'),
    '#submit' => array('ji_quickbooks_connect_submit'),
    '#disabled' => $connect_disabled,
  );

  $disconnect_disabled = FALSE;
  // If realm_id consumer key and secret are not empty then display
  // 'Disconnect from QuickBooks' button.
  if (empty(variable_get('ji_quickbooks_settings_realm_id', '')) ||
      empty(variable_get('ji_quickbooks_settings_consumer_key', '')) ||
      empty(variable_get('ji_quickbooks_settings_consumer_secret', ''))) {
    $disconnect_disabled = TRUE;
  }

  drupal_add_js('https://appcenter.intuit.com/Content/IA/intuit.ipp.anywhere.js', array('type' => 'external'));
  // When the user disconnects from QuickBooks, this also logs them out.
  // This will enable them to enter a new account.
  // Adds a JQuery click() event listener to the disconnect button.
  drupal_add_js(drupal_get_path('module', 'ji_quickbooks') . '/ji_oauth/js/logout.js');

  $form['ji_quickbooks_config']['disconnect'] = array(
    '#type' => 'submit',
    '#value' => t('Disconnect QuickBooks'),
    '#submit' => array('ji_quickbooks_disconnect_submit'),
    '#disabled' => $disconnect_disabled,
  );

  if (!empty(variable_get('ji_quickbooks_settings_realm_id', '')) &&
      !empty(variable_get('ji_quickbooks_settings_consumer_key', '')) &&
      !empty(variable_get('ji_quickbooks_settings_consumer_secret', ''))) {
    $form['ji_quickbooks_config']['manual_renewal'] = array(
      '#type' => 'submit',
      '#value' => t('Manual renewal (NEEDS TESTING)'),
      '#submit' => array('ji_quickbooks_manual_renewal'),
    );
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save settings'),
  );

  return $form;
}

/**
 * Saves settings/parameters to variables.
 */
function ji_quickbooks_admin_main_settings_submit($form = array(), &$form_state = array()) {
  if (isset($form_state['values']['ji_quickbooks'])) {
    variable_set('ji_quickbooks_payment_method', $form_state['values']['ji_quickbooks']['payment_method']);
    variable_set('ji_quickbooks_payment_account', $form_state['values']['ji_quickbooks']['account_payment']);
    variable_set('ji_quickbooks_term', $form_state['values']['ji_quickbooks']['terms']);
    variable_set('ji_quickbooks_default_product', $form_state['values']['ji_quickbooks']['product']);
  }

  $settings = $form_state['input']['ji_quickbooks_config'];
  
  // The user switched between environments, disconnect.
  // Javascript is also enabled to call Intuit's Javascript disconnect
  // see logoff.js.
  if ($settings['environment'] !== variable_get('ji_quickbooks_settings_environment') && $settings['environment'] !== NULL) {
    drupal_set_message(t("Environment variables correctly saved."));
    ji_quickbooks_disconnect_submit();
  }

  variable_set('ji_quickbooks_settings_environment', $settings['environment']);

  drupal_set_message('JI QuickBook settings saved.', 'status', FALSE);
}

/**
 * Submit handler.
 *
 * Starts the OpenID communication process with QBO.
 */
function ji_quickbooks_connect_submit() {
  global $base_url;
  $options = array(
    'query' => array(
      'connectWithIntuitOpenId' => '',
      'return_url' => $base_url . url('admin/config/services/ji_quickbooks'),
      'environment' => variable_get('ji_quickbooks_settings_environment', 'dev'),
  ));
  
  drupal_goto('https://ji-quickbooks.joshideas.com/ji_oauth_authorize', $options);
}

/**
 * Disconnect from QBO.
 *
 * See logoff.js to see Javascript logging off the user from there
 * QBO account as well.
 */
function ji_quickbooks_disconnect_submit() {
  if (variable_get('ji_quickbooks_settings_access_token')) {
    $quickbooks_service = new JIQuickBooksService(FALSE);
    $quickbooks_service->oauthDisconnect();
  }

  // Clear Access token and Access token secret.
  variable_del('ji_quickbooks_settings_access_token');
  variable_del('ji_quickbooks_settings_access_token_secret');
  variable_del('ji_quickbooks_settings_realm_id');
  
  variable_del('ji_quickbooks_settings_consumer_key');
  variable_del('ji_quickbooks_settings_consumer_secret');
}

/**
 * Will manually renew tokens.
 *
 * Still in testing.
 */
function ji_quickbooks_manual_renewal() {
  $quickbooks_service = new JIQuickBooksService(FALSE);
  $quickbooks_service->oauthRenew();
}
